/*
 * Copyright 2003 - 2010 The eFaps Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Revision:        $Rev: 3932 $
 * Last Changed:    $Date: 2010-03-31 15:40:50 -0500 (Wed, 31 Mar 2010) $
 * Last Changed By: $Author: jan.moxter $
 */

package org.efaps.admin.datamodel;

import java.util.Collection;
import java.util.HashSet;
import java.util.StringTokenizer;

/**
 * The class stores the unique keys of a type instance object. One unique
 * key can have one or more attribute.
 *
 * @author The eFaps Team
 * @version $Id: UniqueKey.java 3932 2010-03-31 20:40:50Z jan.moxter $
 */
public class UniqueKey
{
    /**
     * The collection instance variables holds all attributes for this unique
     * key.
     *
     * @see #getAttributes()
     */
    private final Collection<Attribute> attributes = new HashSet<Attribute>();

    /**
     * The constructor creates a new list of attributes which are an unique
     * key for this type.<br/>
     * Each attribute in the  string list is added to this unique key instance.
     *
     * @param _type     related type for the unique key
     * @param _attrList string with comma separated list of attribute names
     */
    UniqueKey(final Type _type,
              final String _attrList)
    {
        final StringTokenizer tokens = new StringTokenizer(_attrList, ",");
        while (tokens.hasMoreTokens())  {
            final Attribute attr = _type.getAttribute(tokens.nextToken());
            attr.addUniqueKey(this);
            getAttributes().add(attr);
        }
    }

    /**
     * This is the getter method for instance variable {@link #attributes}.
     *
     * @return value of instance variable {@link #attributes}
     * @see #attributes
     */
    public Collection<Attribute> getAttributes()
    {
        return this.attributes;
    }
}
