/*
 * Copyright 2003 - 2010 The eFaps Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Revision:        $Rev: 5402 $
 * Last Changed:    $Date: 2010-08-25 19:02:24 -0500 (Wed, 25 Aug 2010) $
 * Last Changed By: $Author: jan.moxter $
 */


package org.efaps.db.search;

import org.efaps.db.AbstractObjectQuery;
import org.efaps.db.wrapper.SQLSelect;
import org.efaps.util.EFapsException;

/**
 * Part of a statement.
 *
 * @author The eFaps Team
 * @version $Id: AbstractQPart.java 5402 2010-08-26 00:02:24Z jan.moxter $
 */
public abstract class AbstractQPart
{

    /**
     * Method adds the sql statement parts to the given StringBuilder.
     *
     * @param _sql StringBuilder to append to
     * @return this AbstractPart
     * @throws EFapsException on any error
     */
    public abstract AbstractQPart appendSQL(final SQLSelect _sql)
        throws EFapsException;

    /**
     * Method is executed to prepare the different parts for execution
     * of the sql statement.
     * @param _query    query the part belong to
     * @param _part     Part this part is nested in
     * @return this AbstractPart
     * @throws EFapsException on any error
     */
    public abstract AbstractQPart prepare(final AbstractObjectQuery<?> _query,
                                          final AbstractQPart _part)
        throws EFapsException;
}
